@@ -6,8 +6,11 @@ from google.protobuf import message as proto_message
 
 from chalk._gen.chalk.common.v1 import chalk_error_pb2, online_query_pb2
 from chalk.client import ChalkError, ChalkException, ErrorCode, ErrorCodeCategory, OnlineQueryContext, QueryMeta
+from chalk.client._internal_models.models import INDEX_COL_NAME, OBSERVED_AT_COL_NAME, PKEY_COL_NAME, TS_COL_NAME
 from chalk.client.models import (
+    BulkOnlineQueryResponse,
     BulkOnlineQueryResult,
+    BulkUploadFeaturesResult,
     FeatureResolutionMeta,
     FeatureResult,
     OnlineQueryManyRequest,
@@ -15,6 +18,8 @@ from chalk.client.models import (
     OnlineQueryResponse,
     OnlineQueryResultFeather,
 )
+from chalk.client.serialization.constants import GRPC_RESULT_METADATA_COL_PREFIX
+from chalk.features._encoding.converter import PrimitiveFeatureConverter
 from chalk.features._encoding.json import FeatureEncodingOptions
 from chalk.parsed._proto.utils import (
     datetime_to_proto_timestamp,
@@ -29,7 +34,6 @@ from chalk.utils.df_utils import arrow_ipc_to_record_batch, pa_table_to_pl_df, r
 
 def check_has_field(proto: proto_message.Message, field: str):
     if not proto.HasField(field):
-
         raise ValueError(f"Proto is missing expected field '{field}'.")
 
 
@@ -42,26 +46,26 @@ def get_field_or(proto: proto_message.Message, field_name: str, default_value: A
 class ChalkErrorConverter:
     _error_code_map: "BidirectionalMap[ErrorCode, chalk_error_pb2.ErrorCode]" = BidirectionalMap(
         [
-            (ErrorCode.INTERNAL_SERVER_ERROR, chalk_error_pb2.ErrorCode.ERROR_CODE_INTERNAL_SERVER_ERROR_UNSPECIFIED),
-            (ErrorCode.PARSE_FAILED, chalk_error_pb2.ErrorCode.ERROR_CODE_PARSE_FAILED),
-            (ErrorCode.RESOLVER_NOT_FOUND, chalk_error_pb2.ErrorCode.ERROR_CODE_RESOLVER_NOT_FOUND),
-            (ErrorCode.INVALID_QUERY, chalk_error_pb2.ErrorCode.ERROR_CODE_INVALID_QUERY),
-            (ErrorCode.VALIDATION_FAILED, chalk_error_pb2.ErrorCode.ERROR_CODE_VALIDATION_FAILED),
-            (ErrorCode.RESOLVER_FAILED, chalk_error_pb2.ErrorCode.ERROR_CODE_RESOLVER_FAILED),
-            (ErrorCode.RESOLVER_TIMED_OUT, chalk_error_pb2.ErrorCode.ERROR_CODE_RESOLVER_TIMED_OUT),
-            (ErrorCode.UPSTREAM_FAILED, chalk_error_pb2.ErrorCode.ERROR_CODE_UPSTREAM_FAILED),
-            (ErrorCode.UNAUTHENTICATED, chalk_error_pb2.ErrorCode.ERROR_CODE_UNAUTHENTICATED),
-            (ErrorCode.UNAUTHORIZED, chalk_error_pb2.ErrorCode.ERROR_CODE_UNAUTHORIZED),
-            (ErrorCode.CANCELLED, chalk_error_pb2.ErrorCode.ERROR_CODE_CANCELLED),
-            (ErrorCode.DEADLINE_EXCEEDED, chalk_error_pb2.ErrorCode.ERROR_CODE_DEADLINE_EXCEEDED),
+            (ErrorCode.INTERNAL_SERVER_ERROR, chalk_error_pb2.ERROR_CODE_INTERNAL_SERVER_ERROR_UNSPECIFIED),
+            (ErrorCode.PARSE_FAILED, chalk_error_pb2.ERROR_CODE_PARSE_FAILED),
+            (ErrorCode.RESOLVER_NOT_FOUND, chalk_error_pb2.ERROR_CODE_RESOLVER_NOT_FOUND),
+            (ErrorCode.INVALID_QUERY, chalk_error_pb2.ERROR_CODE_INVALID_QUERY),
+            (ErrorCode.VALIDATION_FAILED, chalk_error_pb2.ERROR_CODE_VALIDATION_FAILED),
+            (ErrorCode.RESOLVER_FAILED, chalk_error_pb2.ERROR_CODE_RESOLVER_FAILED),
+            (ErrorCode.RESOLVER_TIMED_OUT, chalk_error_pb2.ERROR_CODE_RESOLVER_TIMED_OUT),
+            (ErrorCode.UPSTREAM_FAILED, chalk_error_pb2.ERROR_CODE_UPSTREAM_FAILED),
+            (ErrorCode.UNAUTHENTICATED, chalk_error_pb2.ERROR_CODE_UNAUTHENTICATED),
+            (ErrorCode.UNAUTHORIZED, chalk_error_pb2.ERROR_CODE_UNAUTHORIZED),
+            (ErrorCode.CANCELLED, chalk_error_pb2.ERROR_CODE_CANCELLED),
+            (ErrorCode.DEADLINE_EXCEEDED, chalk_error_pb2.ERROR_CODE_DEADLINE_EXCEEDED),
         ]
     )
 
     _error_category_map: "BidirectionalMap[ErrorCodeCategory, chalk_error_pb2.ErrorCodeCategory]" = BidirectionalMap(
         [
-            (ErrorCodeCategory.NETWORK, chalk_error_pb2.ErrorCodeCategory.ERROR_CODE_CATEGORY_NETWORK_UNSPECIFIED),
-            (ErrorCodeCategory.FIELD, chalk_error_pb2.ErrorCodeCategory.ERROR_CODE_CATEGORY_FIELD),
-            (ErrorCodeCategory.REQUEST, chalk_error_pb2.ErrorCodeCategory.ERROR_CODE_CATEGORY_REQUEST),
+            (ErrorCodeCategory.NETWORK, chalk_error_pb2.ERROR_CODE_CATEGORY_NETWORK_UNSPECIFIED),
+            (ErrorCodeCategory.FIELD, chalk_error_pb2.ERROR_CODE_CATEGORY_FIELD),
+            (ErrorCodeCategory.REQUEST, chalk_error_pb2.ERROR_CODE_CATEGORY_REQUEST),
         ]
     )
 
@@ -108,7 +112,7 @@ class ChalkErrorConverter:
             return category_py
 
     @staticmethod
-    def _chalk_exception_encode(
+    def chalk_exception_encode(
         exception: ChalkException,
     ) -> chalk_error_pb2.ChalkException:
         proto_exception = chalk_error_pb2.ChalkException()
@@ -120,8 +124,8 @@ class ChalkErrorConverter:
         return proto_exception
 
     @staticmethod
-    def _chalk_exception_decode(exception_proto: chalk_error_pb2.ChalkException) -> ChalkException:
-        return ChalkException(
+    def chalk_exception_decode(exception_proto: chalk_error_pb2.ChalkException) -> ChalkException:
+        return ChalkException.create(
             kind=exception_proto.kind,
             message=exception_proto.message,
             stacktrace=exception_proto.stacktrace,
@@ -141,7 +145,7 @@ class ChalkErrorConverter:
         if error.display_primary_key_fqn:
             proto_error.display_primary_key_fqn = error.display_primary_key_fqn
         if error.exception:
-            proto_error.exception.CopyFrom(ChalkErrorConverter._chalk_exception_encode(error.exception))
+            proto_error.exception.CopyFrom(ChalkErrorConverter.chalk_exception_encode(error.exception))
         if error.feature:
             proto_error.feature = error.feature
         if error.resolver:
@@ -153,10 +157,10 @@ class ChalkErrorConverter:
         error_proto: chalk_error_pb2.ChalkError,
     ) -> ChalkError:
         if error_proto.HasField("exception"):
-            chalk_exception = ChalkErrorConverter._chalk_exception_decode(error_proto.exception)
+            chalk_exception = ChalkErrorConverter.chalk_exception_decode(error_proto.exception)
         else:
             chalk_exception = None
-        return ChalkError(
+        return ChalkError.create(
             code=ChalkErrorConverter._chalk_error_code_decode(error_proto.code),
             category=ChalkErrorConverter._chalk_error_code_category_decode(error_proto.category),
             message=error_proto.message,
@@ -168,6 +172,17 @@ class ChalkErrorConverter:
         )
 
 
+class UploadFeaturesBulkConverter:
+    @staticmethod
+    def upload_features_bulk_response_decode(
+        response_proto: online_query_pb2.UploadFeaturesBulkResponse, trace_id: Optional[str]
+    ) -> BulkUploadFeaturesResult:
+        return BulkUploadFeaturesResult(
+            errors=[ChalkErrorConverter.chalk_error_decode(e) for e in response_proto.errors],
+            trace_id=trace_id,
+        )
+
+
 class OnlineQueryConverter:
     @staticmethod
     def _options_bool_field(online_query_context: online_query_pb2.OnlineQueryContext, field: str) -> Optional[bool]:
@@ -264,9 +279,6 @@ class OnlineQueryConverter:
             skip_metrics_storage=OnlineQueryConverter._options_bool_field(
                 online_query_request.context, "skip_metrics_storage"
             ),
-            skip_cache_lookups=OnlineQueryConverter._options_bool_field(
-                online_query_request.context, "skip_cache_lookups"
-            ),
             correlation_id=online_query_request.context.correlation_id,
             query_name=online_query_request.context.query_name,
             query_name_version=online_query_request.context.query_name_version,
@@ -314,7 +326,6 @@ class OnlineQueryConverter:
             "skip_online_storage": request.skip_online_storage,
             "skip_offline_storage": request.skip_offline_storage,
             "skip_metrics_storage": request.skip_metrics_storage,
-            "skip_cache_lookups": request.skip_cache_lookups,
             "store_plan_stages": request.store_plan_stages,
         }
         context_options_dict.update(**(request.planner_options or {}))
@@ -351,7 +362,6 @@ class OnlineQueryConverter:
             "skip_online_storage": request.skip_online_storage,
             "skip_offline_storage": request.skip_offline_storage,
             "skip_metrics_storage": request.skip_metrics_storage,
-            "skip_cache_lookups": request.skip_cache_lookups,
             "store_plan_stages": request.store_plan_stages,
             "planner_version": "2",  # TODO remove this
         }
@@ -453,6 +463,121 @@ class OnlineQueryConverter:
             meta=meta,
         )
 
+    @staticmethod
+    def online_query_bulk_response_decode_to_single(
+        response_proto: online_query_pb2.OnlineQueryBulkResponse,
+    ) -> OnlineQueryResponse:
+        import pyarrow as pa
+
+        oat_prefix = ".__chalk_observed_at__"
+        res: List[FeatureResult] = []
+
+        if len(response_proto.scalars_data) > 0:
+            root_ts: Optional[dt.datetime] = None
+            pkey: Any = None
+            batch: pa.RecordBatch = arrow_ipc_to_record_batch(response_proto.scalars_data)
+            if len(batch) != 1:
+                raise ValueError(f"Expected exactly one scalar data row in response, found {len(batch)}")
+
+            for col_name in batch.schema.names:
+                if col_name.endswith(oat_prefix):
+                    root_ts = batch[col_name][0].as_py()
+                    break
+                if col_name == PKEY_COL_NAME:
+                    pkey = batch[col_name][0].as_py()
+
+            for col_name in batch.schema.names:
+                if (
+                    col_name.startswith("__chalk__.__metadata__.")
+                    or col_name.startswith(GRPC_RESULT_METADATA_COL_PREFIX)
+                    or col_name.endswith(oat_prefix)
+                    or col_name in (TS_COL_NAME, INDEX_COL_NAME, PKEY_COL_NAME, OBSERVED_AT_COL_NAME)
+                ):
+                    continue
+
+                col_field = batch.schema.field(col_name)
+                converter = PrimitiveFeatureConverter(
+                    name=col_name,
+                    is_nullable=col_field.nullable,
+                    pyarrow_dtype=col_field.type,
+                )
+                scoped_ts_fqn = (col_field.metadata or {}).get(b"ts_fqn")
+                if scoped_ts_fqn is not None and scoped_ts_fqn in batch.schema.names:
+                    scoped_ts_val = batch[scoped_ts_fqn][0].as_py()
+                else:
+                    scoped_ts_val = root_ts
+                converted_col = converter.from_pyarrow_to_primitive(batch[col_name])
+                assert (
+                    len(converted_col) == 1
+                ), f"Expected exactly one element in column {col_name}, found {len(converted_col)}"
+                result = FeatureResult(field=col_name, value=converted_col[0], ts=scoped_ts_val, pkey=pkey)
+                metadata_col_name = GRPC_RESULT_METADATA_COL_PREFIX + col_name
+                if metadata_col_name in batch.schema.names:
+                    metadata_col: pa.StructArray = batch[metadata_col_name]
+                    assert isinstance(metadata_col, pa.StructArray)
+                    source_id_field = metadata_col.field("source_id")  # pyright: ignore
+                    source_id: str | None = source_id_field[0].as_py()
+                    source_type_field = metadata_col.field("source_type")  # pyright: ignore
+                    source_type: str | None = source_type_field[0].as_py()
+                    is_cache_hit = source_type in ["online_store", "offline_store"]
+                    resolver_fqn = source_id or "unknown"
+                    if "metadata_val" in metadata_col.type.names:
+                        metadata_col_val_field = metadata_col.field("metadata_val")  # pyright: ignore
+                        metadata_col_val = metadata_col_val_field[0].as_py()
+                        assert (
+                            isinstance(metadata_col_val, int) or metadata_col_val is None
+                        ), f"Expected metadata value, {metadata_col_val}, to be type int or None"
+                        result.metadata_val = metadata_col_val
+                        result.valid = result.metadata_val is not None
+                    else:
+                        # If metadata_val is not returned - these are set to None and "MISSING"
+                        result.valid = (source_id is not None) or (source_type != "MISSING")
+                    if result.valid:
+                        result.meta = FeatureResolutionMeta(
+                            chosen_resolver_fqn=resolver_fqn,
+                            cache_hit=is_cache_hit,
+                            #    version=1
+                        )
+
+                res.append(result)
+
+        for output_name, output_bytes in response_proto.groups_data.items():
+            values = []
+            namespace_ts: Optional[dt.datetime] = None
+            batch = pa.Table.from_batches([arrow_ipc_to_record_batch(output_bytes)])
+
+            for col_name in batch.schema.names:
+                if col_name.endswith(oat_prefix):
+                    namespace_ts = batch[col_name][0].as_py()
+                    break
+
+            for col_name in batch.schema.names:
+                converter = PrimitiveFeatureConverter(
+                    name=col_name,
+                    is_nullable=batch.schema.field(col_name).nullable,
+                    pyarrow_dtype=batch.schema.field(col_name).type,
+                )
+                values.append(converter.from_pyarrow_to_primitive(batch[col_name]))
+
+            res.append(
+                FeatureResult(
+                    field=output_name,
+                    value={
+                        "columns": batch.schema.names,
+                        "values": values,
+                    },
+                    ts=namespace_ts,
+                )
+            )
+
+        return OnlineQueryResponse(
+            data=res,
+            meta=OnlineQueryConverter._query_meta_decode(response_proto.response_meta)
+            if response_proto.HasField("response_meta")
+            else None,
+            errors=[ChalkErrorConverter.chalk_error_decode(e) for e in response_proto.errors],
+        )
+
     @staticmethod
     def online_query_result_feather_encode(
         online_query_result_feather: OnlineQueryResultFeather,
@@ -465,7 +590,7 @@ class OnlineQueryConverter:
         if online_query_result_feather.errors:
             for error in online_query_result_feather.errors:
                 json_error = json.loads(error)
-                chalk_error = ChalkError(**json_error)
+                chalk_error = ChalkError.create(**json_error)
                 chalk_error_proto = chalk_error_pb2.ChalkError()
                 chalk_error_proto.CopyFrom(ChalkErrorConverter.chalk_error_encode(chalk_error))
                 proto_response.errors.append(chalk_error_proto)
@@ -477,7 +602,7 @@ class OnlineQueryConverter:
 
     @staticmethod
     def online_query_bulk_response_decode(
-        response_proto: online_query_pb2.OnlineQueryBulkResponse,
+        response_proto: online_query_pb2.OnlineQueryBulkResponse, trace_id: Optional[str]
     ) -> BulkOnlineQueryResult:
         import polars as pl
         import pyarrow as pa
@@ -496,9 +621,22 @@ class OnlineQueryConverter:
             errors.append(ChalkErrorConverter.chalk_error_decode(err_proto))
         if response_proto.HasField("response_meta"):
             meta = OnlineQueryConverter._query_meta_decode(response_proto.response_meta)
-        return BulkOnlineQueryResult(
-            scalars_df=scalars,
-            groups_dfs=groups,
-            meta=meta,
-            errors=errors,
-        )
+        return BulkOnlineQueryResult(scalars_df=scalars, groups_dfs=groups, meta=meta, errors=errors, trace_id=trace_id)
+
+    @staticmethod
+    def online_query_multi_response_decode(
+        response_proto: online_query_pb2.OnlineQueryMultiResponse, trace_id: Optional[str]
+    ) -> BulkOnlineQueryResponse:
+        results = []
+        for result_proto in response_proto.responses:
+            if result_proto.HasField("single_response"):
+                results.append(OnlineQueryConverter.online_query_response_decode(result_proto.single_response))
+            elif result_proto.HasField("bulk_response"):
+                # Add trace ID to sub-query result objects for clarity
+                results.append(
+                    OnlineQueryConverter.online_query_bulk_response_decode(
+                        result_proto.bulk_response, trace_id=trace_id
+                    )
+                )
+        global_errors = [ChalkErrorConverter.chalk_error_decode(e) for e in response_proto.errors]
+        return BulkOnlineQueryResponse(results=results, global_errors=global_errors, trace_id=trace_id)
