@@ -1,16 +1,16 @@
 from __future__ import annotations
 
-from typing import Any, List, Optional
+from typing import Any, Optional, Sequence
 
 from chalk.features._chalkop import Aggregation, op
 from chalk.features._embedding.embedding import embed
 from chalk.features._encoding.converter import FeatureConverter
 from chalk.features._encoding.missing_value import MissingValueStrategy
-from chalk.features._encoding.primitive import TPrimitive
-from chalk.features._encoding.serialized_dtype import deserialize_dtype, serialize_dtype, serialize_pyarrow_dtype
+from chalk.features._encoding.primitive import TPrimitive, TPrimitiveArrowScalar
+from chalk.features._encoding.serialized_dtype import deserialize_dtype, serialize_dtype
 from chalk.features._vector import Vector
 from chalk.features.dataframe import DataFrame
-from chalk.features.feature_field import Feature, FeatureNotFoundException, feature, has_many, has_one
+from chalk.features.feature_field import CacheStrategy, Feature, FeatureNotFoundException, feature, has_many, has_one
 from chalk.features.feature_set import Features, FeatureSetBase, is_features_cls
 from chalk.features.feature_set_decorator import features
 from chalk.features.feature_time import FeatureTime, feature_time, is_feature_time
@@ -19,9 +19,9 @@ from chalk.features.filter import Filter, TimeDelta, after, before
 from chalk.features.hooks import after_all, before_all
 from chalk.features.primary import Primary, is_primary
 from chalk.features.resolver import Cron, Resolver, ResolverProtocol, offline, online, sink
-from chalk.features.scheduled_query import ScheduledQuery
 from chalk.features.tag import Environments, Tags
-from chalk.features.underscore import Underscore, _, underscore
+from chalk.features.underscore import Underscore, _, __, underscore
+from chalk.queries.scheduled_query import ScheduledQuery  # import to maintain backwards compatibility
 from chalk.utils import MachineType
 
 
@@ -100,7 +100,7 @@ def description(f: Any) -> Optional[str]:
     )
 
 
-def tags(f: Any) -> Optional[List[str]]:
+def tags(f: Any) -> Optional[Sequence[str]]:
     """Get the tags for a feature, feature class, or resolver.
 
     Parameters
@@ -185,6 +185,7 @@ def version(f: Any) -> int | None:
 
 __all__ = [
     "Aggregation",
+    "CacheStrategy",
     "Cron",
     "DataFrame",
     "Environments",
@@ -200,16 +201,20 @@ __all__ = [
     "MissingValueStrategy",
     "Primary",
     "ResolverProtocol",
-    "Underscore",
-    "deserialize_dtype",
+    "ScheduledQuery",
     "TPrimitive",
+    "TPrimitiveArrowScalar",
     "Tags",
     "TimeDelta",
+    "Underscore",
+    "Vector",
     "after",
     "after_all",
     "before",
     "before_all",
     "description",
+    "deserialize_dtype",
+    "embed",
     "ensure_feature",
     "feature",
     "feature_time",
@@ -217,20 +222,17 @@ __all__ = [
     "has_many",
     "has_one",
     "is_feature_time",
-    "is_primary",
     "is_features_cls",
+    "is_primary",
     "offline",
     "online",
     "op",
     "owner",
-    "embed",
     "serialize_dtype",
-    "serialize_pyarrow_dtype",
     "sink",
     "tags",
-    "unwrap_feature",
     "_",
+    "__",
     "underscore",
-    "Vector",
-    "ScheduledQuery",
+    "unwrap_feature",
 ]
