@@ -109,8 +109,21 @@ class op:
         """
         import polars as pl
 
+        from chalk.features import Feature, unwrap_feature
+
         cols = tuple((col, *(t for t in cols)))
+        # FIXME CHA-3207: Support sum(sum(...)) -- i.e. where one of the inputs is an aggregation
+        # Currently allowing the special case where one of the inputs is 0
+
+        cols = tuple(unwrap_feature(x, raise_error=False) for x in cols)
+        cols = tuple(str(x) if isinstance(x, Feature) else x for x in cols)
+
+        cols = tuple(x for x in cols if x != 0)
+        if len(cols) == 0:
+            raise ValueError("Cannot add 0 cols")
         if len(cols) == 1:
+            if isinstance(cols[0], Aggregation):
+                return cols[0]
             return Aggregation(pl.col(str(cols[0])), lambda x: x.sum())
         c = pl.col([str(c) for c in cols])
         if hasattr(pl, "sum_horizontal"):
