@@ -68,3 +68,9 @@ variables `CHALK_API_SERVER` and `CHALK_ENVIRONMENT`.
 """
 
 BranchId: TypeAlias = str
+
+DeploymentId: TypeAlias = str
+"""
+Each `chalk apply` invocation produces a new immutable deployment.
+This type refers to the id of one of these immutable deployments.
+"""
