@@ -3,17 +3,15 @@ from __future__ import annotations
 import collections
 import dataclasses
 import json
-from datetime import timedelta
-from typing import TYPE_CHECKING, Any, List, Optional, Type, TypeVar, Union, overload
+from datetime import datetime, timedelta
+from typing import TYPE_CHECKING, Any, List, Optional, Type, TypeVar, Union, cast, overload
 
 from chalk import Resolver, Windowed
 from chalk.features import DataFrame, Feature, Features, Filter, serialize_dtype
-from chalk.features._encoding.outputs import find_feature_or_resolver_by_fqn
-from chalk.features.feature_field import HasOnePathObj
+from chalk.features.feature_field import HasOnePathObj, VersionInfo
 from chalk.features.feature_set import is_feature_set_class
 from chalk.features.pseudofeatures import PSEUDONAMESPACE
 from chalk.features.resolver import Cron, OfflineResolver, OnlineResolver, SinkResolver, StreamResolver
-from chalk.features.scheduled_query import ScheduledQuery
 from chalk.parsed.duplicate_input_gql import (
     FeatureClassGQL,
     RecomputeFeaturesGQL,
@@ -27,6 +25,7 @@ from chalk.parsed.duplicate_input_gql import (
     UpsertFilterGQL,
     UpsertHasManyKindGQL,
     UpsertHasOneKindGQL,
+    UpsertNamedQueryGQL,
     UpsertReferencePathComponentGQL,
     UpsertResolverGQL,
     UpsertResolverInputUnionGQL,
@@ -37,8 +36,11 @@ from chalk.parsed.duplicate_input_gql import (
     UpsertStreamResolverParamGQL,
     UpsertStreamResolverParamKeyedStateGQL,
     UpsertStreamResolverParamMessageGQL,
+    UpsertWindowMaterializationGQL,
     VersionInfoGQL,
 )
+from chalk.queries.named_query import NamedQuery
+from chalk.queries.scheduled_query import ScheduledQuery
 from chalk.sql._internal.sql_source import BaseSQLSource, TableIngestMixIn
 from chalk.streams.types import (
     StreamResolverParam,
@@ -47,10 +49,16 @@ from chalk.streams.types import (
     StreamResolverParamMessageWindow,
 )
 from chalk.utils import paths
-from chalk.utils.collections import get_unique_item
+from chalk.utils.collections import get_unique_item, unwrap_optional_and_annotated_if_needed
 from chalk.utils.duration import CronTab, Duration, timedelta_to_duration
-from chalk.utils.pydanticutil.pydantic_compat import is_pydantic_basemodel, is_pydantic_basemodel_instance
-from chalk.utils.string import removeprefix, to_snake_case
+from chalk.utils.json import JSON
+from chalk.utils.pydanticutil.pydantic_compat import (
+    get_pydantic_model_json,
+    get_pydantic_output_structure,
+    is_pydantic_basemodel,
+    is_pydantic_basemodel_instance,
+)
+from chalk.utils.string import to_snake_case
 
 if TYPE_CHECKING:
     import pydantic
@@ -75,16 +83,26 @@ def _get_qualified_class_name(cls: Type[Any]):
 
 
 def _convert_type(underlying_type: Type, is_windowed: bool) -> _ConvertedType:
+    if underlying_type is JSON:  # pyright: ignore[reportUnnecessaryComparison]
+        return _ConvertedType(
+            name="chalk.JSON",
+            bases=["object"],
+        )
     if isinstance(underlying_type, Windowed):
         is_windowed = True
         underlying_type = underlying_type.kind
-    bases = []
+
     pretty_name = repr(underlying_type)
-    if isinstance(underlying_type, type):
-        bases.extend(x.__name__ for x in underlying_type.mro())
+    if isinstance(underlying_type, type):  # pyright: ignore[reportUnnecessaryIsInstance]
         pretty_name = underlying_type.__name__
+
+    bases = []
+    actual_underlying_type = unwrap_optional_and_annotated_if_needed(underlying_type)
+    if isinstance(actual_underlying_type, type):  # pyright: ignore[reportUnnecessaryIsInstance]
+        bases.extend(x.__name__ for x in actual_underlying_type.mro())
     else:
         bases = ["object"]
+
     pretty_name = pretty_name.replace("typing.", "")
     if is_windowed:
         bases.append("CHALK_WINDOWED")
@@ -92,11 +110,6 @@ def _convert_type(underlying_type: Type, is_windowed: bool) -> _ConvertedType:
     if isinstance(underlying_type, Feature):
         return _convert_type(underlying_type.converter.rich_type, underlying_type.is_windowed)
 
-    bases = []
-    if isinstance(underlying_type, type):
-        bases.extend(x.__name__ for x in underlying_type.mro())
-    else:
-        bases = ["object"]
     return _ConvertedType(
         name=pretty_name,
         bases=bases,
@@ -157,10 +170,10 @@ def _parse_generic_params_recursively(typ: Any) -> str:
     else:
         type_name = typ.__name__
 
-    type_name = removeprefix(type_name, "typing.")
+    type_name = type_name.removeprefix("typing.")
     if not hasattr(typ, "__args__"):
         return type_name
-    params = [_parse_generic_params_recursively(x) for x in typ.__args__]
+    params = (_parse_generic_params_recursively(x) for x in typ.__args__)
     param_string = ",".join(params)
     return f"{type_name}[{param_string}]"
 
@@ -204,6 +217,11 @@ def convert_type_to_gql(t: ScheduledQuery, path_prefix: Optional[str] = None) ->
     ...
 
 
+@overload
+def convert_type_to_gql(t: NamedQuery, path_prefix: Optional[str] = None) -> UpsertNamedQueryGQL:
+    ...
+
+
 @overload
 def convert_type_to_gql(t: Feature, path_prefix: Optional[str] = None) -> UpsertFeatureGQL:
     ...
@@ -228,8 +246,8 @@ def convert_type_to_gql(
             kind="stream",
             sourceClassName=_get_qualified_class_name(t.source.__class__),
             sourceConfig=t.source.config_to_json(),
-            functionDefinition=t.function_definition,
-            environment=[t.environment] if isinstance(t.environment, str) else t.environment,
+            functionDefinition=t.function_definition or "",
+            environment=None if t.environment is None else list(t.environment),
             doc=t.fn.__doc__,
             machineType=t.machine_type,
             output=[
@@ -250,7 +268,7 @@ def convert_type_to_gql(
     if isinstance(t, SinkResolver):
         return UpsertSinkResolverGQL(
             fqn=t.fqn,
-            functionDefinition=t.function_definition,
+            functionDefinition=t.function_definition or "",
             inputs=[
                 UpsertFeatureReferenceGQL(
                     underlying=_get_feature_id(f, root_fqn=False),
@@ -259,8 +277,8 @@ def convert_type_to_gql(
                 for f in t.inputs
                 if isinstance(f, Feature) and not f.is_autogenerated
             ],
-            environment=t.environment,
-            tags=t.tags,
+            environment=None if t.environment is None else list(t.environment),
+            tags=None if t.tags is None else list(t.tags),
             doc=t.doc,
             machineType=t.machine_type,
             bufferSize=t.buffer_size,
@@ -280,7 +298,11 @@ def convert_type_to_gql(
         inputs = []
         all_inputs = []
         for f in t.inputs:
-            if (not isinstance(f, type) or not issubclass(f, DataFrame)) and not f.is_autogenerated:
+            if (
+                not isinstance(f, type)
+                or not issubclass(f, DataFrame)  # pyright: ignore[reportUnnecessaryIsInstance]  # noqa
+            ) and not f.is_autogenerated:  # pyright: ignore[reportAttributeAccessIssue]
+                f = cast(Feature, f)
                 if f.namespace == PSEUDONAMESPACE:
                     feature_ref = UpsertFeatureReferenceGQL(
                         underlying=_get_pseudofeature_id(f),
@@ -294,7 +316,7 @@ def convert_type_to_gql(
                     )
                     inputs.append(feature_ref)
                     all_inputs.append(UpsertResolverInputUnionGQL(feature=feature_ref))
-            elif isinstance(f, type) and issubclass(f, DataFrame):
+            elif isinstance(f, type) and issubclass(f, DataFrame):  # pyright: ignore[reportUnnecessaryIsInstance]
                 all_inputs.append(UpsertResolverInputUnionGQL(dataframe=_convert_df(f)))
 
         assert t.output is not None
@@ -305,7 +327,7 @@ def convert_type_to_gql(
         return UpsertResolverGQL(
             fqn=t.fqn,
             kind="offline" if isinstance(t, OfflineResolver) else "online",
-            functionDefinition=t.function_definition,
+            functionDefinition=t.function_definition or "",
             inputs=inputs,
             allInputs=all_inputs,
             output=UpsertResolverOutputGQL(
@@ -315,11 +337,13 @@ def convert_type_to_gql(
                     if (not isinstance(f, type) or not issubclass(f, DataFrame)) and not f.is_autogenerated
                 ],
                 dataframes=[
-                    _convert_df(f) for f in t.output.features if isinstance(f, type) and issubclass(f, DataFrame)
+                    _convert_df(cast("type[DataFrame]", f))
+                    for f in t.output.features
+                    if isinstance(f, type) and issubclass(f, DataFrame)
                 ],
             ),
-            environment=t.environment,
-            tags=t.tags,
+            environment=None if t.environment is None else list(t.environment),
+            tags=None if t.tags is None else list(t.tags),
             doc=t.doc,
             cron=convert_type_crontab(cron) if cron is not None else cron,
             machineType=t.machine_type,
@@ -328,6 +352,7 @@ def convert_type_to_gql(
             filename=t.filename if path_prefix is None else t.filename.replace(path_prefix, ""),
             sourceLine=t.source_line,
             dataSources=data_sources,
+            dataLineage=t.data_lineage,
         )
 
     if isinstance(t, StreamResolverParam):
@@ -335,7 +360,7 @@ def convert_type_to_gql(
             converted_type = _convert_type(t.typ, is_windowed=False)
             schema = None
             if is_pydantic_basemodel(t.typ):
-                schema = json.loads(t.typ.schema_json())
+                schema = json.loads(get_pydantic_output_structure(t.typ))
             return UpsertStreamResolverParamGQL(
                 message=UpsertStreamResolverParamMessageGQL(
                     name=t.name,
@@ -350,12 +375,12 @@ def convert_type_to_gql(
             default_value = None
             schema = None
             if dataclasses.is_dataclass(t.default_value):
-                default_value = json.loads(json.dumps(dataclasses.asdict(t.default_value)))
+                default_value = json.loads(json.dumps(dataclasses.asdict(cast(Any, t.default_value))))
                 schema = pydantic.dataclasses.dataclass(t.typ).__pydantic_model__.schema()
             elif is_pydantic_basemodel_instance(t.default_value):
                 assert is_pydantic_basemodel(t.typ)
-                default_value = json.loads(t.default_value.json())
-                schema = json.loads(t.typ.schema_json())
+                default_value = json.loads(get_pydantic_model_json(t.default_value))
+                schema = get_pydantic_output_structure(t.typ)
             return UpsertStreamResolverParamGQL(
                 state=UpsertStreamResolverParamKeyedStateGQL(
                     name=t.name,
@@ -368,12 +393,10 @@ def convert_type_to_gql(
             )
 
     if isinstance(t, ScheduledQuery):
-        for f in t.output:
-            validate_feature_reference(f)
-        get_unique_item((f.split(".")[0] for f in t.output), " of root output feature namespaces")
-        if not isinstance(t.recompute_features, bool):
-            for f in t.recompute_features:
-                validate_feature_reference(f)
+        if t.errors:
+            error_string = "\n".join(f" - {e}" for e in t.errors)
+            raise ValueError(f"Found errors in ScheduledQuery '{t.name}':\n{error_string}")
+        get_unique_item((f.split(".")[0] for f in t.output), "output")
 
         return UpsertCronQueryGQL(
             name=t.name,
@@ -385,13 +408,52 @@ def convert_type_to_gql(
                 featureFqns=list(t.recompute_features) if not isinstance(t.recompute_features, bool) else None,
                 all=t.recompute_features if isinstance(t.recompute_features, bool) else None,
             ),
-            lowerBound=t.lower_bound,
-            upperBound=t.upper_bound,
+            lowerBound=None,
+            upperBound=None,
+            lowerBoundStr=datetime.isoformat(t.lower_bound) if t.lower_bound is not None else None,
+            upperBoundStr=datetime.isoformat(t.upper_bound) if t.upper_bound is not None else None,
             tags=list(t.tags) if t.tags is not None else None,
             requiredResolverTags=list(t.required_resolver_tags) if t.required_resolver_tags is not None else None,
             storeOnline=t.store_online,
             storeOffline=t.store_offline,
             incrementalSources=None if t.incremental_resolvers is None else list(t.incremental_resolvers),
+            resourceGroup=t.resource_group,
+            plannerOptions=t.planner_options,
+        )
+
+    if isinstance(t, NamedQuery):
+        if t.errors:
+            print_name = t.name + "" if t.version is None else f" ({t.version})"
+            error_string = "\n".join(f" - {e}" for e in t.errors)
+            raise ValueError(f"Found errors in NamedQuery '{print_name}':\n{t.code}\n{error_string}")
+        if t.planner_options is not None:
+            try:
+                parsed_planner_options = {k: str(v) for k, v in t.planner_options.items()}
+            except Exception as e:
+                raise ValueError(f"Could not parse provided planner options '{t.planner_options}'") from e
+        else:
+            parsed_planner_options = None
+        if t.filename is None:
+            print_name = t.name + "" if t.version is None else f" ({t.version})"
+            raise ValueError(f"Found errors in NamedQuery '{print_name}':\n failed to find filename")
+
+        return UpsertNamedQueryGQL(
+            name=t.name,
+            queryVersion=t.version,
+            input=t.input,
+            output=t.output,
+            additionalLoggedFeatures=t.additional_logged_features,
+            filename=t.filename if path_prefix is None else t.filename.replace(path_prefix, ""),
+            description=t.description,
+            owner=t.owner,
+            meta=None if t.meta is None else dict(t.meta),
+            tags=list(t.tags) if t.tags is not None else None,
+            staleness=t.staleness,
+            plannerOptions=parsed_planner_options,
+            code=t.code,
+            sourceLineStart=t.source_line_end,
+            sourceLineEnd=t.source_line_start,
+            validPlanNotRequired=t.valid_plan_not_required,
         )
 
     if isinstance(t, Feature):
@@ -411,7 +473,7 @@ def convert_type_to_gql(
                 builder.add_diagnostic(
                     range=builder.annotation_range(t.attribute_name),
                     message=(
-                        f"The attribute '{t.attribute_name}' on the feature class '{t.features_cls.__name__}' "
+                        f"The attribute '{t.features_cls.__name__}.{t.attribute_name}' "
                         f"references {t.typ}. However, the has-one relationship is not defined."
                     ),
                     raise_error=TypeError,
@@ -428,7 +490,7 @@ def convert_type_to_gql(
                 builder.add_diagnostic(
                     range=builder.annotation_range(t.attribute_name),
                     message=(
-                        f"The attribute '{t.attribute_name}' on the feature class '{t.features_cls.__name__}' "
+                        f"The attribute '{t.features_cls.__name__}.{t.attribute_name}' "
                         f"references a DataFrame. However, the has-many relationship is not defined."
                     ),
                     label="Missing has-many join",
@@ -448,7 +510,7 @@ def convert_type_to_gql(
             builder.add_diagnostic(
                 range=builder.annotation_range(t.attribute_name),
                 message=(
-                    f"The attribute '{t.attribute_name}' on the feature class '{t.features_cls.__name__}' "
+                    f"The attribute '{t.features_cls.__name__}.{t.attribute_name}' "
                     f"has a join filter ({t.join}) but its type annotation is not a feature class or "
                     f"DataFrame ({t.typ})."
                 ),
@@ -477,27 +539,59 @@ def convert_type_to_gql(
                     )
                 raise e
             pretty_name = repr(t.converter.rich_type)
-            if isinstance(t.converter.rich_type, type):
+            if isinstance(t.converter.rich_type, type):  # pyright: ignore[reportUnnecessaryIsInstance]
                 pretty_name = _parse_generic_params_recursively(t.converter.rich_type)
             pretty_name = pretty_name.replace("typing.", "")
             if t.is_windowed:
                 pretty_name = f"Windowed[{pretty_name}]"
 
+            feature_version: VersionInfo | None = t.version
+
+            default_json = None
+            if t.default is not Ellipsis:
+                try:
+                    default_json = json.dumps(t.converter.from_rich_to_json(t.default))
+                except:
+                    pass
+            try:
+                serialized_dtype = serialize_dtype(t.converter.pyarrow_dtype)
+            except Exception as e:
+                builder = t.features_cls.__chalk_error_builder__
+                if builder:
+                    builder.add_diagnostic(
+                        message=f"Feature '{t.root_fqn}' has an invalid type: {e.args[0] if len(e.args) > 0 else str(e)}",
+                        range=builder.annotation_range(t.attribute_name) or builder.property_range(t.attribute_name),
+                        code="39",
+                        label="invalid type",
+                        code_href="https://docs.chalk.ai/docs/feature-types",
+                    )
+                raise e
+
             scalar_kind_gql = UpsertScalarKindGQL(
                 scalarKind=pretty_name,
                 primary=t.primary,
                 baseClasses=converted_type.bases,
-                version=t.version and t.version.version,
-                versionInfo=t.version
-                and VersionInfoGQL(
-                    version=t.version.version,
-                    maximum=t.version.maximum,
-                    default=t.version.default,
-                    versions=[f.fqn for f in t.version.reference.values()],
+                version=None if feature_version is None else feature_version.version,
+                versionInfo=None
+                if feature_version is None
+                else VersionInfoGQL(
+                    version=feature_version.version,
+                    maximum=feature_version.maximum,
+                    default=feature_version.default,
+                    versions=[f.fqn for f in feature_version.reference.values()],
                 ),
-                dtype=serialize_dtype(t.converter.pyarrow_dtype),
+                dtype=serialized_dtype,
+                defaultValueJson=default_json,
+                hasExplicitDtype=t._pyarrow_dtype is not None,  # pyright: ignore[reportPrivateUsage]
             )
 
+        window_buckets = None
+        if t.window_buckets is not None:
+            window_buckets = sorted(list(float(x) for x in t.window_buckets))
+        elif t.group_by_windowed is not None:
+            window_buckets = sorted(list(float(x) for x in t.group_by_windowed.buckets_seconds))
+
+        mat = t.window_materialization_parsed
         return UpsertFeatureGQL(
             id=UpsertFeatureIdGQL(
                 fqn=t.fqn,
@@ -508,10 +602,41 @@ def convert_type_to_gql(
                 attributeName=t.attribute_name,
                 explicitNamespace=t.namespace != to_snake_case(t.features_cls.__name__),
             ),
+            windowMaterialization=None
+            if mat is None
+            else UpsertWindowMaterializationGQL(
+                namespace=mat.namespace,
+                groupBy=[
+                    UpsertFeatureReferenceGQL(
+                        underlying=_get_feature_id(f, root_fqn=False),
+                        path=[_get_path_component(p) for p in f.path or []],
+                    )
+                    for f in mat.group_by
+                ],
+                bucketDuration=float(mat.bucket_duration_seconds),
+                aggregation=mat.aggregation,
+                aggregateOn=UpsertFeatureReferenceGQL(
+                    underlying=_get_feature_id(mat.aggregate_on, root_fqn=False),
+                    path=[_get_path_component(p) for p in t.path or []],
+                )
+                if mat.aggregate_on is not None
+                else None,
+                dtype=serialize_dtype(mat.pyarrow_dtype),
+                backfillResolver=mat.backfill_resolver,
+                backfillLookbackDuration=float(mat.backfill_lookback_duration_seconds)
+                if mat.backfill_lookback_duration_seconds is not None
+                else None,
+                backfillStartTime=mat.backfill_start_time,
+                backfillSchedule=mat.backfill_schedule,
+                continuousResolver=mat.continuous_resolver,
+                continuousBufferDuration=float(mat.continuous_buffer_duration_seconds)
+                if mat.continuous_buffer_duration_seconds is not None
+                else None,
+            ),
             maxStaleness=timedelta_to_duration(t.max_staleness),
             description=t.description,
             owner=t.owner,
-            windowBuckets=None if t.window_buckets is None else sorted(list(t.window_buckets)),
+            windowBuckets=window_buckets,
             etlOfflineToOnline=t.etl_offline_to_online,
             tags=None if t.tags is None else list(t.tags),
             hasOneKind=has_one_kind_gql,
@@ -532,6 +657,9 @@ def convert_type_to_gql(
         )
 
     if isinstance(t, Filter):
+        while isinstance(t.lhs, Filter):
+            t = t.lhs
+
         if isinstance(t.lhs, Feature):
             assert (
                 t.lhs.is_scalar or t.lhs.is_feature_time
@@ -555,15 +683,6 @@ def convert_type_crontab(cron: Union[CronTab, Duration]) -> str:
     return f"{cron.total_seconds()}s" if isinstance(cron, timedelta) else cron
 
 
-def validate_feature_reference(f: str):
-    path = f.split(".")
-    fqn = ".".join(path[: min(2, len(path))])
-    if (feature := find_feature_or_resolver_by_fqn(fqn)) is None:
-        raise ValueError(f"Feature or resolver '{f}' not found.")
-    if isinstance(feature, Resolver):
-        raise ValueError(f"'{f}' is a resolver, not a feature.")
-
-
 def gather_cdc_sources() -> List[UpsertCDCSourceGQL]:
     integration_name_to_tables = collections.defaultdict(list)
     for source in BaseSQLSource.registry:
