@@ -672,6 +672,7 @@ class ToProtoConverter:
                 join=ToProtoConverter.convert_filter(f.join),
                 foreign_namespace=f.joined_class.namespace,
                 max_staleness_duration=max_staleness_duration,
+                online_store_max_items=f.online_store_max_items,
                 tags=f.tags,
                 owner=f.owner,
                 description=f.description,
@@ -1048,6 +1049,8 @@ class ToProtoConverter:
             resource_hint = pb.RESOURCE_HINT_CPU
         elif r.resource_hint == "io":
             resource_hint = pb.RESOURCE_HINT_IO
+        elif r.resource_hint == "gpu":
+            resource_hint = pb.RESOURCE_HINT_GPU
         else:
             raise ValueError(f"Unsupported resource hint: {r.resource_hint}")
 
