@@ -1,5 +1,7 @@
 from typing import Any, List, Literal, Mapping, Optional, Union
 
+from typing_extensions import final
+
 from chalk.features import Feature
 from chalk.sql.finalized_query import DataframeFinalizedChalkQuery, Finalizer, SingletonFinalizedChalkQuery
 from chalk.sql.protocols import BaseSQLSourceProtocol, ChalkQueryProtocol, IncrementalSettings
@@ -7,6 +9,7 @@ from chalk.utils.duration import Duration, parse_chalk_duration
 from chalk.utils.missing_dependency import missing_dependency_exception
 
 
+@final
 class ChalkQuery(ChalkQueryProtocol):
     def __init__(
         self,
@@ -23,6 +26,12 @@ class ChalkQuery(ChalkQueryProtocol):
         self._features = features
         self._source = source
 
+    @classmethod
+    def __init_subclass__(cls, **kwargs: Any):
+        raise RuntimeError(
+            "The ChalkQuery class should not be subclassed. The Chalk runtime assumes that this class is final."
+        )
+
     def first(self, params: Optional[Mapping[str, Any]] = None) -> SingletonFinalizedChalkQuery:
         from sqlalchemy.sql import Select
 
@@ -94,9 +103,7 @@ class ChalkQuery(ChalkQueryProtocol):
             finalizer=Finalizer.ALL,
             source=self._source,
             incremental_settings=IncrementalSettings(
-                lookback_period=(
-                    parse_chalk_duration(lookback_period) if isinstance(lookback_period, str) else lookback_period
-                ),
+                lookback_period=parse_chalk_duration(lookback_period),
                 incremental_column=None if incremental_column is None else str(incremental_column),
                 mode=mode,
                 incremental_timestamp=incremental_timestamp,
