@@ -1,9 +1,9 @@
 from __future__ import annotations
 
 import os
-from typing import TYPE_CHECKING, Any, Dict, Optional, Union
+from typing import TYPE_CHECKING, Any, Dict, Mapping, Optional, Union
 
-from chalk.integrations.named import load_integration_variable
+from chalk.integrations.named import create_integration_variable, load_integration_variable
 from chalk.sql._internal.sql_source import BaseSQLSource, SQLSourceKind, TableIngestMixIn
 from chalk.sql.protocols import SQLSourceWithTableIngestProtocol
 from chalk.utils.missing_dependency import missing_dependency_exception
@@ -12,6 +12,13 @@ if TYPE_CHECKING:
     from sqlalchemy.engine.url import URL
 
 
+_MYSQL_HOST_NAME = "MYSQL_HOST"
+_MYSQL_TCP_PORT_NAME = "MYSQL_TCP_PORT"
+_MYSQL_DATABASE_NAME = "MYSQL_DATABASE"
+_MYSQL_USER_NAME = "MYSQL_USER"
+_MYSQL_PWD_NAME = "MYSQL_PWD"
+
+
 class MySQLSourceImpl(BaseSQLSource, TableIngestMixIn, SQLSourceWithTableIngestProtocol):
     def __init__(
         self,
@@ -23,6 +30,7 @@ class MySQLSourceImpl(BaseSQLSource, TableIngestMixIn, SQLSourceWithTableIngestP
         name: Optional[str] = None,
         engine_args: Optional[Dict[str, Any]] = None,
         async_engine_args: Optional[Dict[str, Any]] = None,
+        integration_variable_override: Optional[Mapping[str, str]] = None,
     ):
         try:
             import pymysql
@@ -30,15 +38,25 @@ class MySQLSourceImpl(BaseSQLSource, TableIngestMixIn, SQLSourceWithTableIngestP
             raise missing_dependency_exception("chalkpy[mysql]")
         del pymysql
         self.name = name
-        self.host = host or load_integration_variable(name="MYSQL_HOST", integration_name=name)
+        self.host = host or load_integration_variable(
+            name=_MYSQL_HOST_NAME, integration_name=name, override=integration_variable_override
+        )
         self.port = (
             int(port)
             if port is not None
-            else load_integration_variable(name="MYSQL_TCP_PORT", integration_name=name, parser=int)
+            else load_integration_variable(
+                name=_MYSQL_TCP_PORT_NAME, integration_name=name, parser=int, override=integration_variable_override
+            )
+        )
+        self.db = db or load_integration_variable(
+            name=_MYSQL_DATABASE_NAME, integration_name=name, override=integration_variable_override
+        )
+        self.user = user or load_integration_variable(
+            name=_MYSQL_USER_NAME, integration_name=name, override=integration_variable_override
+        )
+        self.password = password or load_integration_variable(
+            name=_MYSQL_PWD_NAME, integration_name=name, override=integration_variable_override
         )
-        self.db = db or load_integration_variable(name="MYSQL_DATABASE", integration_name=name)
-        self.user = user or load_integration_variable(name="MYSQL_USER", integration_name=name)
-        self.password = password or load_integration_variable(name="MYSQL_PWD", integration_name=name)
         self.ingested_tables: Dict[str, Any] = {}
         if engine_args is None:
             engine_args = {}
@@ -85,3 +103,16 @@ class MySQLSourceImpl(BaseSQLSource, TableIngestMixIn, SQLSourceWithTableIngestP
             port=self.port,
             database=self.db,
         )
+
+    def _recreate_integration_variables(self) -> Dict[str, str]:
+        return {
+            k: v
+            for k, v in [
+                create_integration_variable(_MYSQL_HOST_NAME, self.name, self.host),
+                create_integration_variable(_MYSQL_TCP_PORT_NAME, self.name, self.port),
+                create_integration_variable(_MYSQL_DATABASE_NAME, self.name, self.db),
+                create_integration_variable(_MYSQL_USER_NAME, self.name, self.user),
+                create_integration_variable(_MYSQL_PWD_NAME, self.name, self.password),
+            ]
+            if v is not None
+        }
