@@ -15,7 +15,7 @@ import pyarrow.parquet as pq
 
 import chalk.logging
 from chalk.features import Feature, FeatureConverter
-from chalk.integrations.named import load_integration_variable
+from chalk.integrations.named import create_integration_variable, load_integration_variable
 from chalk.sql._internal.integrations.util import json_parse_and_cast
 from chalk.sql._internal.query_execution_parameters import QueryExecutionParameters
 from chalk.sql._internal.sql_source import (
@@ -29,7 +29,6 @@ from chalk.utils.df_utils import is_binary_like, read_parquet
 from chalk.utils.environment_parsing import env_var_bool
 from chalk.utils.log_with_context import get_logger
 from chalk.utils.missing_dependency import missing_dependency_exception
-from chalk.utils.string import removesuffix
 from chalk.utils.threading import DEFAULT_IO_EXECUTOR
 
 if TYPE_CHECKING:
@@ -94,6 +93,15 @@ _BOTO_CLIENT_LOCK = threading.Lock()
 _BOTO_SERIALIZE_CALLS: bool = env_var_bool("CHALK_REDSHIFT_SERIALIZE_BOTO_CALLS")
 
 
+_REDSHIFT_HOST_NAME = "REDSHIFT_HOST"
+_REDSHIFT_DB_NAME = "REDSHIFT_DB"
+_REDSHIFT_USER_NAME = "REDSHIFT_USER"
+_REDSHIFT_PASSWORD_NAME = "REDSHIFT_PASSWORD"
+_REDSHIFT_UNLOAD_IAM_ROLE_NAME = "REDSHIFT_UNLOAD_IAM_ROLE"
+_REDSHIFT_PORT_NAME = "REDSHIFT_PORT"
+_REDSHIFT_UNLOAD_S3_BUCKET_NAME = "REDSHIFT_UNLOAD_S3_BUCKET"
+
+
 class RedshiftSourceImpl(BaseSQLSource):
     kind = SQLSourceKind.redshift
 
@@ -105,10 +113,12 @@ class RedshiftSourceImpl(BaseSQLSource):
         password: Optional[str] = None,
         name: Optional[str] = None,
         port: Optional[Union[int, str]] = None,
+        unload_iam_role: Optional[str] = None,
         engine_args: Optional[Dict[str, Any]] = None,
         s3_client: Optional[S3Client] = None,
         s3_bucket: Optional[str] = None,
         executor: Optional[ThreadPoolExecutor] = None,
+        integration_variable_override: Optional[Mapping[str, str]] = None,
     ):
         try:
             import boto3
@@ -116,23 +126,36 @@ class RedshiftSourceImpl(BaseSQLSource):
         except ImportError:
             raise missing_dependency_exception("chalkpy[redshift]")
         del redshift_connector
-        self.host = host or load_integration_variable(name="REDSHIFT_HOST", integration_name=name)
-        self.db = db or load_integration_variable(name="REDSHIFT_DB", integration_name=name)
-        self.user = user or load_integration_variable(name="REDSHIFT_USER", integration_name=name)
-        self.password = password or load_integration_variable(name="REDSHIFT_PASSWORD", integration_name=name)
+        self.host = host or load_integration_variable(
+            name=_REDSHIFT_HOST_NAME, integration_name=name, override=integration_variable_override
+        )
+        self.db = db or load_integration_variable(
+            name=_REDSHIFT_DB_NAME, integration_name=name, override=integration_variable_override
+        )
+        self.user = user or load_integration_variable(
+            name=_REDSHIFT_USER_NAME, integration_name=name, override=integration_variable_override
+        )
+        self.password = password or load_integration_variable(
+            name=_REDSHIFT_PASSWORD_NAME, integration_name=name, override=integration_variable_override
+        )
+        self.unload_iam_role = unload_iam_role or load_integration_variable(
+            name=_REDSHIFT_UNLOAD_IAM_ROLE_NAME, integration_name=name, override=integration_variable_override
+        )
         self.port = (
             int(port)
             if port is not None
-            else load_integration_variable(name="REDSHIFT_PORT", integration_name=name, parser=int)
+            else load_integration_variable(
+                name=_REDSHIFT_PORT_NAME, integration_name=name, parser=int, override=integration_variable_override
+            )
         )
         # TODO customer may want to provide the s3 creds via the "AWS Integration" we provide.
         # NOTE: Chalk Engine injects its own S3 client via `_DEFAULT_REDSHIFT_S3_CLIENT`
         self._s3_client = s3_client or _DEFAULT_REDSHIFT_S3_CLIENT or boto3.client("s3")
         self._s3_bucket = s3_bucket or load_integration_variable(
-            name="REDSHIFT_UNLOAD_S3_BUCKET", integration_name=name
+            name=_REDSHIFT_UNLOAD_S3_BUCKET_NAME, integration_name=name, override=integration_variable_override
         )
         if self._s3_bucket is not None:
-            self._s3_bucket = removesuffix(self._s3_bucket, "s3://").lstrip("/")
+            self._s3_bucket = self._s3_bucket.removesuffix("s3://").lstrip("/")
         self._executor = executor or DEFAULT_IO_EXECUTOR
 
         if engine_args is None:
@@ -156,6 +179,10 @@ class RedshiftSourceImpl(BaseSQLSource):
         engine_args.setdefault("isolation_level", os.environ.get("CHALK_SQL_ISOLATION_LEVEL", "AUTOCOMMIT"))
         BaseSQLSource.__init__(self, name=name, engine_args=engine_args, async_engine_args={})
 
+    @property
+    def s3_bucket(self):
+        return self._s3_bucket
+
     def convert_db_types(self, v: Any, converter: FeatureConverter):
         """
         Redshift returns binary data as a hex string. Need to convert to bytes before returning as feature data.
@@ -180,6 +207,9 @@ class RedshiftSourceImpl(BaseSQLSource):
             port=self.port,
         )
 
+    def supports_inefficient_fallback(self) -> bool:
+        return False
+
     def _execute_query_efficient(
         self,
         finalized_query: FinalizedChalkQuery,
@@ -219,6 +249,7 @@ class RedshiftSourceImpl(BaseSQLSource):
                 assert isinstance(execution_context, self.get_engine().dialect.execution_ctx_cls)
                 operation = execution_context.statement
                 assert operation is not None
+                operation = operation.strip().removesuffix(";")
                 params = execution_context.parameters[0]
                 unload_destination = None
 
@@ -231,15 +262,20 @@ class RedshiftSourceImpl(BaseSQLSource):
                     _logger.debug(f"Executing query & creating temp table '{temp_table_name}'")
                     cursor.execute(f"CREATE TEMP TABLE {temp_table_name} AS ({operation})", params)
                 except Exception as e:
+                    _public_logger.error(f"Failed to create temp table for operation: {operation}", exc_info=e)
                     raise RuntimeError(f"Failed to create temp table for operation: {operation}") from e
                 else:
                     unload_destination = f"{temp_table_name}/"
                     unload_uri = f"s3://{self._s3_bucket}/{unload_destination}"
-                    unload_query = f"UNLOAD ('SELECT * FROM {temp_table_name}') TO '{unload_uri}' IAM_ROLE default FORMAT PARQUET EXTENSION 'parquet'"
+                    unload_iam_role = "default" if self.unload_iam_role is None else f"'{self.unload_iam_role}'"
+                    unload_query = f"UNLOAD ('SELECT * FROM {temp_table_name}') TO '{unload_uri}' IAM_ROLE {unload_iam_role} FORMAT PARQUET EXTENSION 'parquet'"
                     _public_logger.info(f"Executing UNLOAD query [{temp_query_id}]: {unload_query}")
                     cursor.execute(unload_query)
                 finally:
-                    cursor.execute(f"DROP TABLE {temp_table_name}")
+                    try:
+                        cursor.execute(f"DROP TABLE {temp_table_name}")
+                    except Exception:
+                        _logger.warning(f"Failed to drop temp table '{temp_table_name}'", exc_info=True)
         # Redshift is case-insensitive with column names, so let's map it back to what we were expecting
         assert unload_destination is not None
         _public_logger.info(
@@ -261,6 +297,7 @@ class RedshiftSourceImpl(BaseSQLSource):
         filenames = list(_list_files(self._s3_client, self._s3_bucket, unload_destination))
         download_futs: list[Future[pa.Table]] = []
         for filename in filenames:
+            _logger.debug("Scheduling download of file %s", filename)
             fut = self._executor.submit(
                 _download_file_to_table, self._s3_client, self._s3_bucket, filename, columns_to_features
             )
@@ -287,6 +324,21 @@ class RedshiftSourceImpl(BaseSQLSource):
 
         register_redshift_compiler_hooks()
 
+    def _recreate_integration_variables(self) -> dict[str, str]:
+        return {
+            k: v
+            for k, v in [
+                create_integration_variable(_REDSHIFT_HOST_NAME, self.name, self.host),
+                create_integration_variable(_REDSHIFT_DB_NAME, self.name, self.db),
+                create_integration_variable(_REDSHIFT_USER_NAME, self.name, self.user),
+                create_integration_variable(_REDSHIFT_PASSWORD_NAME, self.name, self.password),
+                create_integration_variable(_REDSHIFT_PORT_NAME, self.name, self.port),
+                create_integration_variable(_REDSHIFT_UNLOAD_IAM_ROLE_NAME, self.name, self.unload_iam_role),
+                create_integration_variable(_REDSHIFT_UNLOAD_S3_BUCKET_NAME, self.name, self._s3_bucket),
+            ]
+            if v is not None
+        }
+
 
 def _boto_lock_ctx() -> typing.ContextManager:
     if _BOTO_SERIALIZE_CALLS:
@@ -296,8 +348,7 @@ def _boto_lock_ctx() -> typing.ContextManager:
     return lock_ctx
 
 
-# TODO move chalkshared.utils.storage_client into chalkpy
-def _list_files(client: "S3Client", bucket: str, prefix: str) -> Iterable[str]:
+def _list_files(client: S3Client, bucket: str, prefix: str) -> Iterable[str]:
     try:
         continuation_token = None
         while True:
@@ -327,7 +378,7 @@ def _list_files(client: "S3Client", bucket: str, prefix: str) -> Iterable[str]:
 
 
 def _download_file_to_table(
-    client: "S3Client",
+    client: S3Client,
     bucket: str,
     filename: str,
     cols_to_features: Callable[[Sequence[str]], Mapping[str, Feature]],
