@@ -58,3 +58,6 @@ class SQLiteSourceImpl(TableIngestMixIn, BaseSQLSource, SQLSourceWithTableIngest
 
     def get_sqlglot_dialect(self) -> Union[str, None]:
         return "sqlite"
+
+    def _recreate_integration_variables(self) -> dict[str, str]:
+        return {}
