@@ -2,6 +2,8 @@ from __future__ import annotations
 
 from typing import TYPE_CHECKING, Any, Literal, Mapping, Optional, Union
 
+from typing_extensions import final
+
 from chalk.features import Feature
 from chalk.sql import IncrementalSettings
 from chalk.sql.finalized_query import DataframeFinalizedChalkQuery, Finalizer, SingletonFinalizedChalkQuery
@@ -13,6 +15,7 @@ if TYPE_CHECKING:
     from sqlalchemy.sql.elements import TextClause
 
 
+@final
 class StringChalkQuery(StringChalkQueryProtocol):
     def __init__(
         self,
@@ -31,6 +34,12 @@ class StringChalkQuery(StringChalkQueryProtocol):
         self._params = params
         self._fields = {k: Feature.from_root_fqn(v) if isinstance(v, str) else v for (k, v) in fields.items()}
 
+    @classmethod
+    def __init_subclass__(cls, **kwargs: Any):
+        raise RuntimeError(
+            "The StringChalkQuery class should not be subclassed. The Chalk runtime assumes that this class is final."
+        )
+
     def __repr__(self):
         return f"StringChalkQuery(query='{self._query}')"
 
@@ -96,9 +105,7 @@ class StringChalkQuery(StringChalkQueryProtocol):
             params=self._params,
             finalizer=Finalizer.ALL,
             incremental_settings=IncrementalSettings(
-                lookback_period=(
-                    parse_chalk_duration(lookback_period) if isinstance(lookback_period, str) else lookback_period
-                ),
+                lookback_period=parse_chalk_duration(lookback_period) if lookback_period is not None else None,
                 incremental_column=None if incremental_column is None else incremental_column,
                 mode=mode,
                 incremental_timestamp=incremental_timestamp,
